/*----------------------------------------------------------------------------
 Copyright:      Jens Mundhenke
 Author:         Jens Mundhenke
 Remarks:		 Used on an simplified IP-stack of Ulrich Radig
 				 Collection of eeprom functions that were spreaded over the project
 known Problems:
 Version:        18.01.2009
 Description:    EEPROM-functions for the IP stack and applications

 This modules has a set of functions to read/write data from/to EEPROM. It uses
 the WIN AVR EEPROM functions, but gives some extra functionality as default
 values. Some functions are copied from Ulrichs original code, the idea was to
 collect EEPROM functions in one module.

 The functions have read/write features for
  - IP adresses (array of 4 ints in, 1 long with default out)
  - strings (null terminated char array with max length, default if nothing is stored)
  - char
  - block (array of chars with fixed length)

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der
 GNU General Public License, wie von der Free Software Foundation verffentlicht,
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
 (nach Ihrer Option) jeder spteren Version.

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung,
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 Programm erhalten haben.
 Falls nicht, schreiben Sie an die Free Software Foundation,
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
----------------------------------------------------------------------------*/

#include <avr/eeprom.h>
#include <string.h>

#include "config.h"
#include "para.h"

//----------------------------------------------------------------------------
// para_getip - reading an IP address from EEPROM, if there is one != 0xFFFFFFFF
//              or returning the default-value
unsigned long para_getip (unsigned char *eeprom_address,unsigned long default_value)
{
	unsigned char value[4];

	for (unsigned char count = 0; count<4;count++)
	{
		//eeprom_busy_wait ();
		value[count] = eeprom_read_byte(eeprom_address + count);
	}

	//Ist der EEPROM Inhalt leer?
	if ((*((unsigned long*)&value[0])) == 0xFFFFFFFF)
	{
		return(default_value);
	}
	return((*((unsigned long*)&value[0])));
}


//------------------------------------------------------------------------------
// para_setip - stores an array of 4 int(!) as chars to EEPROM
void para_setip (unsigned char *eeprom_address, unsigned int ip[])
{
	if (ip[0] != 0x00000000)
	{
		//value ins EEPROM schreiben
		for (unsigned char count = 0; count<4;count++)
		{
			eeprom_busy_wait ();
			eeprom_write_byte(eeprom_address + count,ip[count]);
		}
	}
}


//----------------------------------------------------------------------------
// para_getstring - reading a string parameter from EEPROM, if there is one
//                  until the string is terminated by 0 or 0xFF or the len is reached
//                  if there is no string (0xFF before 0) or an unterminated string
//					copy the default value as result.
// WARNING: the length of the default string is not checked against length of destination
void para_getstring (char *name, unsigned char *eeprom_address, char len, char *default_value)
{
	int i;
	char c;
	i = 0;

	// copy string from eeprom to memory
	do {
		c = eeprom_read_byte (eeprom_address+i);
		name[i] = c;
	} while ( (c != 0) && (c != 0xFF) && (i++ < len) );

	// if there is empty eeprom or no termination, copy default value
	if ( (c == 0xFF) || (i > len) )
		strcpy ( name, default_value );
}


//------------------------------------------------------------------------------
// para_setstring - writes string s to EEPROM, if the string is longer than max_len
//                  it is truncated
void para_setstring (unsigned char *eeprom_address, unsigned char max_len, char *s)
{
	int len = 0;

	if ( !s || !(*s) )	// leeren String ignorieren
		return;

	do {
		//value ins EEPROM schreiben
		eeprom_busy_wait ();
		eeprom_write_byte(eeprom_address + len, *s);
		len++;
	} 	while (*(s++) && (len < max_len-1));

	// terminate string for sure
	eeprom_busy_wait ();
	eeprom_write_byte(eeprom_address + max_len - 1, 0x00);
}


//------------------------------------------------------------------------------
// para_getchar - reads a char from EEPROM
unsigned char para_getchar (unsigned char *eeprom_address)
{
	return ( eeprom_read_byte (eeprom_address) );
}


//------------------------------------------------------------------------------
// para_setchar - stores char c to EEPROM
void para_setchar (unsigned char *eeprom_address, unsigned char c)
{
	eeprom_busy_wait ();
	eeprom_write_byte(eeprom_address, c);
}


//----------------------------------------------------------------------------
// para_getblock - reading a parameter block from EEPROM
//                 the contents is not checked
void para_getblock (unsigned char *eeprom_address, char *block, unsigned int len)
{
	for ( unsigned int i=0; i<len; i++, block++ ) {
		*block = eeprom_read_byte (eeprom_address+i);
	}
}


//------------------------------------------------------------------------------
// para_setblock - writing a parameter block to EEPROM
//                 the contents is not checked
void para_setblock (unsigned char *eeprom_address, char *block, unsigned int len)
{
	for ( unsigned int i=0; i<len; i++, block++ ) {
		eeprom_busy_wait ();
		eeprom_write_byte(eeprom_address+i, *block);
	}
}



